/*****************************************************************************/
/*                                                                           */
/*  Copyright Notice: IBM Personal Communication/3270 Version 4.2            */
/*                    (C) COPYRIGHT IBM CORP. 1997 - PROGRAM PROPERTY        */
/*                    OF IBM ALL RIGHTS RESERVED                             */
/*                                                                           */
/*****************************************************************************/
//
// This sample application can be built with IBM VisualAge C++ using the
// MAKEFILE provided, or with Microsoft Visual C++ V4.2 Developer Studio
// using the supplied project workspace file (ECLMON.MDP).
//
// Updated for PComm Version 4.3: Changed to use new PS update event on the
// ECLPS object, and OIA event on the OIA object (rather than the combined
// events on the ECLSession object).
//
//-----------------------------------------------------------------------------
//
// This ECL sample application implements a simple "system monitor".  It
// displays the status of all PComm sessions in a color-coded matrix.  Any
// session can be selected and the contents of that session's screen will be
// displayed.  Tabs on the notebook are used to examine various aspects of
// the currently selected session (e.g. examine properties of individual
// fields, connection characteristics, etc).  All pages of the notebook are
// updated dynamically as the session data changes, and the status matrix
// is updated dynamically as sessions start, stop, connect, and disconnect.
//
// This sample demonstrates one possible use of almost all the ECL objects
// and events.  Some basic design points to help understand this application:
//
// -- Only one copy of the event objects are created.  The same object is
//    reused over and over to handle events on different sessions.
//
// -- There is only one ECLSession object pointer.  When the user changes
//    sessions by clicking on a status button, the current session object
//    is deleted.  A new object is then created for the selected session.
//    All the other objects (PS, OIA, etc.) are retrieved from the session
//    object.
//
// -- The "Session" object pointer is a global variable.  It is only
//    accessed by the main window thread -- it is not accessed directly
//    by the event objects (which run on a different thread), so we 
//    don't have to worry about serializing access to it.
//
// -- The event objects execute as quickly as possible.  When they need
//    to have work done, they post a message to the appropriate window
//    and continue.  Thus all the work is done on the main thread and the
//    events run as quickly as possible.
//
// -- The status matrix is an array of owner-draw buttons with a series
//    of IDs from IDC_SESSOIN1 through IDC_SESSION1+25.
//
// -- The main window is a dialog, so WinMain just runs a simple
//    DialogBox() call to run the application.
//
// -- The ConnList[] array is an array of data structures, one for each
//    possible PComm session.  It is fixed allocated at 26 since that is
//    the max number of sessions currently supported by PComm (V4.2).  
//    This fixed array would not be a good idea for supporting an
//    arbitrary number of sessions.  "CurrConn" is the index into this
//    array of the currently selected session.
//
// -- Each tab of the notebook is a seperate dialog with its own dialog
//    procedure.  The tab dialogs are notified when events occure (such as
//    the session disconnecting, or the presentation space being updated).
//    "CurrTab" is the index of the currently displayed tab.
//
//-----------------------------------------------------------------------------

#define STRICT
#include <windows.h>        // Windows header
#include <commctrl.h>       // Common controls
#include <process.h>        // Thread support
#include <stdio.h>          // C language header
#include "resource.h"
#include "eclall.hpp"       // Emulator Class Library

//---------------------------------------------------------------------------------------------
// File Scope Variable Declarations
//---------------------------------------------------------------------------------------------

//Application instance handle
HINSTANCE hInst;

#define CONNSTATUS_STOPPED 0         // Status codes stored in ConnList[] array
#define CONNSTATUS_READY   1
#define CONNSTATUS_DISCONN 2
#define CONNSTATUS_NOAPI   3

// Application defined window messages
#define CHANGED_CONN       WM_APP+0  // Sent to each dialog tab -- changed connection or status
#define CHANGED_PS         WM_APP+1  // Sent to each dialog tab -- PS updated
#define CHANGED_OIA        WM_APP+2  // Sent to each dialog tab -- OIA updated
#define CHANGED_STATUS     WM_APP+3  // Send to main dialog -- status of conn changed
#define CLEAR_FIELDINFO    WM_APP+4  // Used internally by FIELD dialog
#define STARTEVENT_ERROR   WM_APP+5  // Posted by start/stop event handler
#define UPDATE_CONNINFO    WM_APP+6  // Used internally by CONNECTION dialog

static HFONT MonoFont;            // Monospaced font for PS display
static HFONT SmallFont;           // Small font
//#define MAX_TABS 5                // Number of tabs on the notebook control
#define MAX_TABS 3
static TC_ITEM TabList[MAX_TABS] = {
  {TCIF_TEXT, 0, 0, "Screen",     sizeof("Screen"),        -1, NULL},
  {TCIF_TEXT, 0, 0, "Fields",     sizeof("Fields"),        -1, NULL},
  {TCIF_TEXT, 0, 0, "Connection", sizeof("Connection"),    -1, NULL},
  // These items are not implemented in this sample
//{TCIF_TEXT, 0, 0, "Metrics",    sizeof("Metrics"),       -1, NULL},
//{TCIF_TEXT, 0, 0, "File Xfer",  sizeof("File Xfer"),     -1, NULL},
  };

typedef struct _ConnInfo {        // Structure describing one connection
  ECLConnection *ConnObj;         // Ptr to ECLConnection object
  long          ConnHandle;       // Handle of the connection
  HWND          StatusButton;     // Handle of status window
  int           ConnStatus;       // Last known status (CONNSTATUS_* value)
} ConnInfo;

ConnInfo ConnList[26];            // Array of all connection descriptions

static ECLConnMgr *ConnMgr;       // Connection Manager object
static ECLSession *Session=NULL;  // Session object for current connection
static HWND MainHwnd;             // Main window handle
static HWND TabHwnd;              // Main window tab control handle
static int  CurrTab;              // Index of currently displayed tab
static int  CurrConn;             // Current index into ConnList array

//---------------------------------------------------------------------------------------------
// Helper macros (assume dialog handle is "hwnd")
//---------------------------------------------------------------------------------------------
#define ENABLECONTROL(id) EnableWindow(GetDlgItem(hwnd, id), TRUE)
#define DISABLECONTROL(id) EnableWindow(GetDlgItem(hwnd, id), FALSE)
#define FOCUSCONTROL(id) SetFocus(GetDlgItem(hwnd, id))
#define SETCHECK(id) SendMessage(GetDlgItem(hwnd, id), BM_SETCHECK, (WPARAM)BST_CHECKED, 0)
#define CLEARCHECK(id) SendMessage(GetDlgItem(hwnd, id), BM_SETCHECK, (WPARAM)BST_UNCHECKED, 0)
#define SETTEXT(id,str) SetWindowText(GetDlgItem(hwnd, id), str)
#define GETTEXT(id,str,len) GetWindowText(GetDlgItem(hwnd,id),str,len)
#define GETINT(id, i) i=GetDlgItemInt(hwnd, id, NULL, FALSE)
#define GETLONG(id, i) i=((long)GetDlgItemInt(hwnd, id, NULL, FALSE))
#define QUERYCHECK(id) ((BOOL)SendMessage(GetDlgItem(hwnd, id), BM_GETCHECK, 0, 0))
#define CLEARLIST(id)  SendDlgItemMessage(hwnd, id, LB_RESETCONTENT, 0, 0)
#define QUERYSELECTION(id) SendDlgItemMessage(hwnd, id, LB_GETCURSEL, 0, 0)
#define ERRBOX(h,title,msg) MessageBoxA(h, msg, title, MB_OK|MB_ICONERROR)
#define CLR_BLACK                       RGB(0,0,0)
#define CLR_DARKBLUE                    RGB(0,0,128)
#define CLR_DARKGREEN                   RGB(0,128,0)
#define CLR_DARKCYAN                    RGB(0,128,128)
#define CLR_DARKRED                     RGB(128,0,0)
#define CLR_DARKPINK                    RGB(128,0,128)
#define CLR_BROWN                       RGB(128,128,0)
#define CLR_PALEGRAY                    RGB(192,192,192)
#define CLR_DARKGRAY                    RGB(128,128,128)
#define CLR_BLUE                        RGB(0,0,255)
#define CLR_GREEN                       RGB(0,255,0)
#define CLR_CYAN                        RGB(0,255,255)
#define CLR_RED                         RGB(255,0,0)
#define CLR_PINK                        RGB(255,0,255)
#define CLR_YELLOW                      RGB(255,255,0)
#define CLR_WHITE                       RGB(255,255,255)

// Macro to post a message to all the notebook pages
#define POSTTOALL(msg,mp1,mp2) {int i; \
                                for (i=0; i<MAX_TABS; i++) \
                                  PostMessage((HWND)TabList[i].lParam, msg, (WPARAM)mp1, (LPARAM)mp2); \
                               }


//---------------------------------------------------------------------------------------------
// Prototypes
//---------------------------------------------------------------------------------------------
BOOL WINAPI DlgMainProc(HWND hwnd, UINT uMessage, WPARAM wParam, LPARAM lParam);
void DrawSessionButton(DRAWITEMSTRUCT *DrawData, LONG Color, LONG ColorBorder, char SessID);
BOOL WINAPI ScreenDlgProc(HWND hwnd, UINT msg, WPARAM mp1, LPARAM mp2);
BOOL WINAPI FieldsDlgProc(HWND hwnd, UINT msg, WPARAM mp1, LPARAM mp2);
BOOL WINAPI ConnDlgProc(HWND hwnd, UINT msg, WPARAM mp1, LPARAM mp2);
BOOL WINAPI MetricsDlgProc(HWND hwnd, UINT msg, WPARAM mp1, LPARAM mp2);
BOOL WINAPI XferDlgProc(HWND hwnd, UINT msg, WPARAM mp1, LPARAM mp2);
BOOL WINAPI DlgStartConnection(HWND hwnd, UINT msg, WPARAM mp1, LPARAM mp2);

//---------------------------------------------------------------------------------------------
// Connection Start/Stop Event Handler
//---------------------------------------------------------------------------------------------

// Define a class derivied from ECLStartNotify

class MyStartEvents: public ECLStartNotify
{
public:
  // We only override these two functions
  void	NotifyEvent(ECLConnMgr *CMObj, long ConnHandle, BOOL Started);
  void  NotifyError(ECLConnMgr *CMObj, long ConnHandle, ECLErr ErrObj);
};

// Implement the overriden class functions

void MyStartEvents::NotifyEvent(ECLConnMgr *CMObj, long ConnHandle, BOOL Started)
{
  // This function is called whenever a connection starts or stops.

  // Tell the main window to update the status display
  PostMessage(MainHwnd, CHANGED_STATUS, 0, (LPARAM)ConnHandle);
}

void MyStartEvents::NotifyError(ECLConnMgr *CMObj, long ConnHandle, ECLErr ErrObj)
{
  // This function is called whenever an error occures during
  // connection start/stop.  Note this might happen a long while
  // after we tried to start or stop a new connection.  We will
  // be called here if, for example, an invalid profile name is
  // given when we try to start a new connection.

  ECLErr *ErrCopy;

  ErrCopy = new ECLErr(ErrObj);  // Make a copy to pass

  // Tell the main window and pass a pointer to the error object.
  // The main window will delete the copy.
  PostMessage(MainHwnd, STARTEVENT_ERROR, 0, (LPARAM)ErrCopy);
}

//---------------------------------------------------------------------------------------------
// Comm Link Event Handler
//---------------------------------------------------------------------------------------------

// Define a class derivied from ECLCommNotify

class MyCommEvents: public ECLCommNotify
{
public:
  // We only override this one function
  void	NotifyEvent(ECLConnection *ConnObj, BOOL Connected);
};

// Implement the overriden class function

void MyCommEvents::NotifyEvent(ECLConnection *ConnObj, BOOL Connected)
{
  // This function is called whenever the comm link changes state.
  PostMessage(MainHwnd, CHANGED_STATUS, 0, (LPARAM)ConnObj->GetHandle());

}

//---------------------------------------------------------------------------------------------
// PS Updated Event Handler
//---------------------------------------------------------------------------------------------

// Define a class derivied from ECLPSNotify

class MyPSEvents: public ECLPSNotify
{
public:
  // We only override this one function
  void	NotifyEvent(ECLPS *PSObj);
};

// Implement the overriden class function

void MyPSEvents::NotifyEvent(ECLPS *PSObj)
{

  // This function is called whenever the PS changes.

  // Make sure the connection on which this happended is still the
  // current session (could have changed while we were getting
  // this notification).
  if (PSObj->GetHandle() != ConnList[CurrConn].ConnHandle)
    return;

  // Post messages to all the dialog pages
  POSTTOALL(CHANGED_PS, 0, 0);
}

//---------------------------------------------------------------------------------------------
// OIA Updated Event Handler
//---------------------------------------------------------------------------------------------

// Define a class derivied from ECLOIANotify

class MyOIAEvents: public ECLOIANotify
{
public:
  // We only override this one function
  void	NotifyEvent(ECLOIA *OIAObj);
};

// Implement the overriden class function

void MyOIAEvents::NotifyEvent(ECLOIA *OIAObj)
{

  // This function is called whenever the OIA changes.

  // Make sure the connection on which this happended is still the
  // current session (could have changed while we were getting
  // this notification).
  if (OIAObj->GetHandle() != ConnList[CurrConn].ConnHandle)
    return;

  // Post messages to all the dialog pages
  POSTTOALL(CHANGED_OIA, 0, 0);
}

//---------------------------------------------------------------------------------------------
// Keystroke Event Handler
//---------------------------------------------------------------------------------------------

// Define a class derivied from ECLKeyNotify

class MyKeyEvents: public ECLKeyNotify
{
public:
  // We only override this one function
  int	NotifyEvent(ECLPS *PSObj, char const KeyType[2], const char * const KeyString);
};

// Implement the overriden class function

int MyKeyEvents::NotifyEvent(ECLPS *PSObj, char const KeyType[2], const char * const KeyString)
{
#define KEY_PROCESS 0
#define KEY_FILTER  1

  // This function is called whenever the user presses a key
  // in the current connection.

  // Make sure the connection on which this happended is still the
  // current session (could have changed while we were getting
  // this notification).
  if (PSObj->GetHandle() != ConnList[CurrConn].ConnHandle)
    return KEY_PROCESS;

  // Just display the keystroke on the SCREEN tab dialog page and continue
  // to process it normally.
  SetWindowText(GetDlgItem((HWND)TabList[0].lParam, IDC_KEYMON), KeyString);
  return KEY_PROCESS;

}

MyStartEvents *StartEvents;    // Ptr to my start/stop event handler
MyCommEvents *CommEvents;      // Ptr to my comm link event handler
MyPSEvents *PSEvents;          // Ptr to my PS update event handler
MyOIAEvents *OIAEvents;        // Ptr to my OIA update event handler
MyKeyEvents  *KeyEvents;       // Ptr to my keystroke event handler

//---------------------------------------------------------------------------------------------
// Main dialog window procedure
//---------------------------------------------------------------------------------------------
BOOL WINAPI DlgMainProc(HWND hwnd, UINT msg, WPARAM mp1, LPARAM mp2)
{
  int i;

  try { // Capture any misc ECL errors

  switch (msg) {

  case WM_INITDIALOG: {
    RECT Rect, RectDlg;

    TabHwnd = GetDlgItem(hwnd, IDC_TAB);  // Keep some key window handles
    MainHwnd = hwnd;

    // Create main window tabs setting instance data to dlg window handles
    TabList[0].lParam = (long)CreateDialog(hInst, MAKEINTRESOURCE(IDD_SCREEN), TabHwnd, ScreenDlgProc);
    TabList[1].lParam = (long)CreateDialog(hInst, MAKEINTRESOURCE(IDD_FIELDS), TabHwnd, FieldsDlgProc);
    TabList[2].lParam = (long)CreateDialog(hInst, MAKEINTRESOURCE(IDD_CONN),   TabHwnd, ConnDlgProc);
  // These items are not implemented in this sample
//    TabList[3].lParam = (long)CreateDialog(hInst, MAKEINTRESOURCE(IDD_METRICS),TabHwnd, MetricsDlgProc);
//    TabList[4].lParam = (long)CreateDialog(hInst, MAKEINTRESOURCE(IDD_XFER),   TabHwnd, XferDlgProc);
    for (i=0; i<MAX_TABS; i++) {
      SendMessage(TabHwnd, TCM_INSERTITEM, (WPARAM)i, (LPARAM)&(TabList[i]));
    }

    // Get info about tab control including it's display area
    GetWindowRect(TabHwnd, &Rect);
    OffsetRect(&Rect, -Rect.left, -Rect.top);
    SendMessage(TabHwnd, TCM_ADJUSTRECT, (WPARAM)FALSE, (LPARAM)&Rect);

    for (i=0; i<MAX_TABS; i++) {
      // Move dialogs into the tab display area and size to fit
      GetWindowRect(TabHwnd, &RectDlg);
      SetWindowPos((HWND)TabList[i].lParam, HWND_TOP, 
        Rect.left,
        Rect.top,
        Rect.right-Rect.left,
        Rect.bottom-Rect.top,
        SWP_NOACTIVATE|SWP_NOZORDER);
    }

    // Select tab 0 to start
    CurrTab = 0;
    ShowWindow((HWND)TabList[0].lParam, SW_SHOW);

    // Create my notifcation object handlers
    StartEvents = new MyStartEvents();  // Connection start/stop
    CommEvents  = new MyCommEvents();   // Comm link connect/disconnect
    PSEvents    = new MyPSEvents();     // PS updates
    OIAEvents   = new MyOIAEvents();    // OIA updates
    KeyEvents   = new MyKeyEvents();    // Keystroke events

    // Create connection manager and register for start/stop events
    ConnMgr = new ECLConnMgr();
    ConnMgr->RegisterStartEvent(StartEvents);

    // Build ConnList array of connection information and register
    // for comm link events on each one.  Note same event object is
    // used for all connections.  Initial event will cause comm status
    // change of 'A' connection which will trigger rest of initialization.
    CurrConn = 0;
    memset(ConnList, 0x00, sizeof(ConnList));
    for (i=0; i<26; i++) {
      ConnList[i].StatusButton = GetDlgItem(hwnd, IDC_SESSION1+i);
      ConnList[i].ConnObj = new ECLConnection((char)('A'+i));
      ConnList[i].ConnHandle = ConnList[i].ConnObj->GetHandle();
      ConnList[i].ConnObj->RegisterCommEvent(CommEvents, TRUE);
    }

    break;
  }

  case WM_DESTROY:     
    // Cleanup any current session object
    if (Session != NULL) {
      //Session->UnregisterUpdateEvent(UpdtEvents);
      Session->GetPS()->UnregisterKeyEvent(KeyEvents);
      Session->GetPS()->UnregisterPSEvent(PSEvents);
      Session->GetOIA()->UnregisterOIAEvent(OIAEvents);
      delete Session;
      Session = NULL;
    }

    // Cleanup the connection manager
    ConnMgr->UnregisterStartEvent(StartEvents);
    delete ConnMgr;

    // Cleanup all the connection objects
    for (i=0; i<26; i++) {
      ConnList[i].ConnObj->UnregisterCommEvent(CommEvents);
      delete ConnList[i].ConnObj;
    }

    // Cleanup event handlers
    delete StartEvents;
    delete CommEvents ;
    delete PSEvents ;
    delete OIAEvents;
    delete KeyEvents  ;

    break;

  case WM_NOTIFY:
    switch (((NMHDR *)mp2)->idFrom) {
    case IDC_TAB:  //------------------- Notification from tab control
      switch (((NMHDR *)mp2)->code) {
      case TCN_SELCHANGE: //------------ User changed tabs, swap dialogs
        ShowWindow((HWND)TabList[CurrTab].lParam, SW_HIDE);
        CurrTab = (int)SendMessage(TabHwnd, TCM_GETCURSEL, 0, 0);
        ShowWindow((HWND)TabList[CurrTab].lParam, SW_SHOW);
      }
    }
    break;

  case CHANGED_STATUS: 
    // The status of a connection has changed.  mp2 is the handle of
    // the connection.  We will update the visual status display.
    int Index;

    for (Index=0; Index<26; Index++) {  // Look for connection that changed
      if ((long)mp2 == ConnList[Index].ConnHandle) {  // Found it
        if (ConnList[Index].ConnObj->IsReady() == TRUE) 
          ConnList[Index].ConnStatus = CONNSTATUS_READY;
        else if (!ConnList[Index].ConnObj->IsStarted()) 
          ConnList[Index].ConnStatus = CONNSTATUS_STOPPED;
        else if (!ConnList[Index].ConnObj->IsCommStarted())
          ConnList[Index].ConnStatus = CONNSTATUS_DISCONN;
        else 
          ConnList[Index].ConnStatus = CONNSTATUS_NOAPI;

        // Update the visual display
        InvalidateRect(ConnList[Index].StatusButton, NULL, TRUE);

        // If the current connection was the one that changed, fake a
        // click on the connection button to (re)create the Session
        // object or delete it as needed for the new status.

        if (Index == CurrConn)
          SendMessage(hwnd, WM_COMMAND, (WPARAM)IDC_SESSION1+CurrConn, 0);

        break;
      }
    }
    return 0;  // CHANGED_STATUS

  case WM_COMMAND: 

    // First, look for any of the session status buttons

    if ((LOWORD(mp1) >= IDC_SESSION1) && (LOWORD(mp1) <= IDC_SESSION1+26-1)) {
      // Connection button was selected... switch connections
      int OldConn;

      OldConn = CurrConn;
      CurrConn = LOWORD(mp1) - IDC_SESSION1;

      // Repaint buttons w/updated inset emphasis
      InvalidateRect(ConnList[OldConn].StatusButton, NULL, TRUE);
      InvalidateRect(ConnList[CurrConn].StatusButton, NULL, TRUE);

      // Cleanup Session object for prior connection
      if (Session != NULL) {
        //Session->UnregisterUpdateEvent(UpdtEvents);
        Session->GetPS()->UnregisterPSEvent(PSEvents);
        Session->GetOIA()->UnregisterOIAEvent(OIAEvents);
        delete Session;
        Session = NULL;
      }

      // Create new session object only if connection is READY
      if (ConnList[CurrConn].ConnStatus == CONNSTATUS_READY) {
        Session = new ECLSession(ConnList[CurrConn].ConnHandle);
        try {
          //Session->RegisterUpdateEvent(PSOIAUpdate, UpdtEvents, TRUE);
          Session->GetPS()->RegisterPSEvent(PSEvents, TRUE);
          Session->GetOIA()->RegisterOIAEvent(OIAEvents, TRUE);
        } catch (ECLErr Err) {
          ERRBOX(hwnd,  "ECL Error registering for update events:", Err.GetMsgText());
        }
      }

      // Setup connection control buttons based on current status
      switch (ConnList[CurrConn].ConnStatus) {
      case CONNSTATUS_READY:
        ENABLECONTROL(IDC_START);
        ENABLECONTROL(IDC_CONNECT);
        SETTEXT(IDC_START, "Stop");
        SETTEXT(IDC_CONNECT, "Disconn");
        break;
      case CONNSTATUS_STOPPED:
        ENABLECONTROL(IDC_START);
        DISABLECONTROL(IDC_CONNECT);
        SETTEXT(IDC_START, "Start");
        SETTEXT(IDC_CONNECT, "Connect");
        break;
      case CONNSTATUS_DISCONN:
        ENABLECONTROL(IDC_START);
        ENABLECONTROL(IDC_CONNECT);
        SETTEXT(IDC_START, "Stop");
        SETTEXT(IDC_CONNECT, "Connect");
        break;
      case CONNSTATUS_NOAPI:
        DISABLECONTROL(IDC_START);
        DISABLECONTROL(IDC_CONNECT);
        SETTEXT(IDC_START, "Start");
        SETTEXT(IDC_CONNECT, "Connect");
        break;
      }

      // Tell current dialog page about the change (others will find
      // out when they become visible).
      PostMessage((HWND)TabList[CurrTab].lParam, CHANGED_CONN, 0, 0);
      //POSTTOALL(CHANGED_CONN, 0, 0);

      return 0;
    }

    // Other normal buttons on the main dialog

    switch (LOWORD(mp1)) {
    case IDC_START: //----------------- Start/Stop button
      if (ConnList[CurrConn].ConnObj->IsStarted()) {
        ConnMgr->StopConnection(ConnList[CurrConn].ConnHandle, "");
      } else {
        DialogBox(hInst, MAKEINTRESOURCE(IDD_STARTCONNECTION), hwnd, (DLGPROC)DlgStartConnection);
      }
      return 0;
    case IDC_CONNECT: //--------------- Connect/Disconnect button
      if (ConnList[CurrConn].ConnObj->IsCommStarted())
        ConnList[CurrConn].ConnObj->StopCommunication();
      else
        ConnList[CurrConn].ConnObj->StartCommunication();
    }

    return FALSE;

  case WM_DRAWITEM: { /* Draw color session buttons */
    DRAWITEMSTRUCT *DrawData;
    ULONG           BorderColor, FillColor;

    DrawData = (DRAWITEMSTRUCT *)mp2;
    if (DrawData->itemState & ODS_FOCUS)
      BorderColor = CLR_BLACK;
      else BorderColor = CLR_DARKGRAY;

    i = mp1 - IDC_SESSION1;            // Index into ConnList array
    switch (ConnList[i].ConnStatus) {
    case CONNSTATUS_READY:             // Select color based on status
      FillColor = CLR_GREEN;
      break;
    case CONNSTATUS_STOPPED:
      FillColor = CLR_PALEGRAY;
      break;
    case CONNSTATUS_DISCONN:
      FillColor = CLR_YELLOW;
      break;
    default:
      FillColor = CLR_RED;
      break;
    }

    // Draw the custom button face
    DrawSessionButton(DrawData, FillColor, BorderColor, 'A'+i);

    return TRUE;
  }

  case STARTEVENT_ERROR:
    // Error interface of start/stop event handler was called.
    // mp2 is a pointer to an ECLErr object.  We just display
    // the error message.
    ERRBOX(hwnd,"Connection start/stop error:", ((ECLErr *)mp2)->GetMsgText());
    delete (ECLErr *)mp2;
    return 0;

  case WM_CLOSE:
    EndDialog(hwnd, 0);
    PostQuitMessage(0);
    break;


  } // switch msg

} catch (ECLErr Err) {
  ERRBOX(hwnd,  "ECL Error (main dialog):", Err.GetMsgText());
}

  return FALSE;
  
}

//---------------------------------------------------------------------------------------------
BOOL WINAPI ScreenDlgProc(HWND hwnd, UINT msg, WPARAM mp1, LPARAM mp2)
//---------------------------------------------------------------------------------------------
// Dialog procedure for the "Screen" tab of the notebook
//---------------------------------------------------------------------------------------------
{

  try { // Capture any misc ECL errors

  switch (msg) {

  case WM_INITDIALOG: {
    // Set PS display to small monospaced font
    SendDlgItemMessage(hwnd, IDC_PSTEXT, WM_SETFONT, (WPARAM)MonoFont, (LPARAM)TRUE);
    break;
    }

  case CHANGED_CONN: {
    // Changed connections (or status of current connection).
    // If there is no Session object, disable everything.

    if (Session == NULL) {
      DISABLECONTROL(IDC_SENDKEYS);
      DISABLECONTROL(IDC_SEND);
      SETTEXT(IDC_SENDKEYS, "");
      SETTEXT(IDC_KEYMON, "");
      DISABLECONTROL(IDC_OIAINHIBIT);
      DISABLECONTROL(IDC_OIAINSERT);
      DISABLECONTROL(IDC_OIACAPS);
      DISABLECONTROL(IDC_OIANUM);
      DISABLECONTROL(IDC_OIAAPL);
      DISABLECONTROL(IDC_OIACAPSLOCK);
      DISABLECONTROL(IDC_OIADBCS);
      DISABLECONTROL(IDC_OIACOMMERR);
      DISABLECONTROL(IDC_OIAMSGWAIT);

      switch (ConnList[CurrConn].ConnStatus) {
      case CONNSTATUS_DISCONN:
        SETTEXT(IDC_PSTEXT, "<CONNECTION DISCONNECTED>");
        break;
      case CONNSTATUS_NOAPI:
        SETTEXT(IDC_PSTEXT, "<CONNECTION API DISABLED>");
        break;
      case CONNSTATUS_STOPPED:
        SETTEXT(IDC_PSTEXT, "<CONNECTION IS STOPPED>");
        break;
      default:
        SETTEXT(IDC_PSTEXT, "");
        break;
      }
    } else { 
      // Enable everything.  We will get PS and OIA updates
      // when the event is registered to update those fields.
      ENABLECONTROL(IDC_SENDKEYS);
      ENABLECONTROL(IDC_SEND);
      SETTEXT(IDC_KEYMON, "");
      SETTEXT(IDC_SENDKEYS, "");
      SendMessage(hwnd, WM_COMMAND, IDC_MONITOR, 0); // Set key monitor if checked
    }
    return 0;
  } // msg CHANGED_CONN

  case WM_SHOWWINDOW:
    if ((BOOL)mp1 == TRUE) { // This dialog becoming visible
      // Force update
      SendMessage(hwnd, CHANGED_CONN, 0, 0);
      if (Session != NULL) {
        SendMessage(hwnd, CHANGED_OIA, 0, 0);
        SendMessage(hwnd, CHANGED_PS,  0, 0);
      }
    }
    break;


  case CHANGED_OIA: {
    // OIA has been updated... update visual status
    ULONG   Status;

    if (CurrTab != 0)         // Ignore if not displaying this dialog
      return 0; 

    // We get all the status flags in one call which is much
    // faster than making a call for each one.
    Status = Session->GetOIA()->GetStatusFlags();

    switch (Status & OIAFLAG_INHIBMASK) {
    case NotInhibited:
      SETTEXT(IDC_OIAINHIBIT, "Uninhib");
      ENABLECONTROL(IDC_OIAINHIBIT);
      break;
    case SystemWait:
      SETTEXT(IDC_OIAINHIBIT, "System");
      ENABLECONTROL(IDC_OIAINHIBIT);
      break;
    case CommCheck:
      SETTEXT(IDC_OIAINHIBIT, "CommChk");
      ENABLECONTROL(IDC_OIAINHIBIT);
      break;
    case ProgCheck:
      SETTEXT(IDC_OIAINHIBIT, "ProgChk");
      ENABLECONTROL(IDC_OIAINHIBIT);
      break;
    case MachCheck:
      SETTEXT(IDC_OIAINHIBIT, "MachChk");
      ENABLECONTROL(IDC_OIAINHIBIT);
      break;
    case OtherInhibit:
      SETTEXT(IDC_OIAINHIBIT, "X");
      ENABLECONTROL(IDC_OIAINHIBIT);
      break;
    default:
      SETTEXT(IDC_OIAINHIBIT, "?");
      ENABLECONTROL(IDC_OIAINHIBIT);
      break;
    } // Inhibit switch

    // Update static text controls based on OIA status
    if (Status & OIAFLAG_INSERT)
      ENABLECONTROL(IDC_OIAINSERT);
    else
      DISABLECONTROL(IDC_OIAINSERT);

    if (Status & OIAFLAG_UPSHIFT)
      ENABLECONTROL(IDC_OIACAPS);
    else
      DISABLECONTROL(IDC_OIACAPS);

    if (Status & OIAFLAG_NUMERIC)
      ENABLECONTROL(IDC_OIANUM);
    else
      DISABLECONTROL(IDC_OIANUM);

    if (Status & OIAFLAG_APL)
      ENABLECONTROL(IDC_OIAAPL);
    else
      DISABLECONTROL(IDC_OIAAPL);

    if (Status & OIAFLAG_CAPSLOCK)
      ENABLECONTROL(IDC_OIACAPSLOCK);
    else
      DISABLECONTROL(IDC_OIACAPSLOCK);

    if (Status &OIAFLAG_DBCS)
      ENABLECONTROL(IDC_OIADBCS);
    else
      DISABLECONTROL(IDC_OIADBCS);

    if (Status & OIAFLAG_COMMERR)
      ENABLECONTROL(IDC_OIACOMMERR);
    else
      DISABLECONTROL(IDC_OIACOMMERR);

    if (Status & OIAFLAG_MSGWAIT)
      ENABLECONTROL(IDC_OIAMSGWAIT);
    else
      DISABLECONTROL(IDC_OIAMSGWAIT);

  } // OIA changed
  return 0;

  case CHANGED_PS: {
    // PS has been updated... update visual PS display
    ECLPS  *PS;               // Ptr to current session PS object
    ULONG  i;

    char *Screen;             // Character plane buffer
    ULONG Size;               // Size of PS, plus 3
    ULONG LineLen, Rows;      // Dimensions of the PS
    char Save1, Save2, Save3; // Temp character stores
    char *Ptr;                // Ptr into screen buffer

    if (CurrTab != 0)         // Ignore if this dialog is not visible
      return 0; 

    PS = Session->GetPS();                    // Get ptr to PS object
    PS->GetSize(&Rows, &LineLen);             // Get dimensions of PS
    Size = (Rows * LineLen) + 3;              // Calc reqd buffer size

    Screen = new char[Size];                  // Allocate buffer
    PS->GetScreen(Screen, Size, TextPlane);   // Get text plane

    // For each line, terminate it w/CR+LF+NULL and insert
    // into the multiline edit box control.

    SETTEXT(IDC_PSTEXT, "");                  // Clear edit box
    for (Ptr = Screen, i=0;                   // Start on 1st row
         i < Rows;                            // Until end of screen
         Ptr = Ptr+LineLen, i++) {            // Move ptr and counter

      Save1 = *(Ptr+LineLen);                 // Save 1st 3 chars of next row
      Save2 = *(Ptr+LineLen+1);
      Save3 = *(Ptr+LineLen+2);

      strcpy(Ptr+LineLen, "\r\n");            // Terminate line w/CR+LF+NULL
      SendDlgItemMessage(hwnd, IDC_PSTEXT, EM_REPLACESEL, FALSE, (LPARAM)Ptr);

      *(Ptr+LineLen) = Save1;                 // Restore next row
      *(Ptr+LineLen+1) = Save2;
      *(Ptr+LineLen+2) = Save3;
    } // for each line

    delete []Screen;                          // Cleanup buffer
  }  // msg CHANGED_PS

  case WM_COMMAND: 
    switch (LOWORD(mp1)) {
    case IDC_SEND: // Send keystrokes to the current connection, catch errors
      char SendBuff[255];
      GETTEXT(IDC_SENDKEYS, SendBuff, sizeof(SendBuff));
      try {
        Session->GetPS()->SendKeys(SendBuff);
      } catch (ECLErr Err) {
        ERRBOX(hwnd, "ECL Error sending keystrokes:", Err.GetMsgText());
      }
      break;

    case IDC_MONITOR: // Toggle keystroke monitor
      SETTEXT(IDC_KEYMON, "");
      if (Session == NULL)
        break;
      if (QUERYCHECK(IDC_MONITOR))
        Session->GetPS()->RegisterKeyEvent(KeyEvents);
      else
        Session->GetPS()->UnregisterKeyEvent(KeyEvents);
      break;

    }
    return FALSE;


  } // Switch on msg

} catch (ECLErr Err) {
  ERRBOX(hwnd,  "ECL Error (screen tab):", Err.GetMsgText());
}

  return FALSE;
}

//---------------------------------------------------------------------------------------------
BOOL WINAPI FieldsDlgProc(HWND hwnd, UINT msg, WPARAM mp1, LPARAM mp2)
//---------------------------------------------------------------------------------------------
// Dialog procedure for the "Fields" tab of the notebook.
//---------------------------------------------------------------------------------------------
{
  int i;
  ECLField      *Fld;         // Ptr to one field object

  switch (msg) {

  case WM_INITDIALOG: {
    // Set column width of multicolumn listbox for 4 columns
    RECT Rect;

    GetWindowRect(GetDlgItem(hwnd, IDC_FIELDLIST), &Rect);
    SendDlgItemMessage(hwnd, IDC_FIELDLIST, LB_SETCOLUMNWIDTH, (Rect.right-Rect.left)/4, 0);

    break;
    }

  case WM_SHOWWINDOW:
    if ((BOOL)mp1 == TRUE) { // We are becoming visible
      SendMessage(hwnd, CHANGED_PS, 0, 0);  // Refresh list
    }
    break;

  case CLEAR_FIELDINFO: {
    // Reset all the GUI controls to empty values
    CLEARLIST(IDC_FIELDLIST);       
    SETTEXT(IDC_FLDSTARTROW, "");
    SETTEXT(IDC_FLDENDROW, "");
    SETTEXT(IDC_FLDSTARTCOL, "");
    SETTEXT(IDC_FLDENDCOL, "");
    SETTEXT(IDC_FLDLEN, "");
    SETTEXT(IDC_FIELDTEXT, "");
    CLEARCHECK(IDC_FLDMOD);
    CLEARCHECK(IDC_FLDPROTECT);
    CLEARCHECK(IDC_FLDHIGH);
    CLEARCHECK(IDC_FLDDISPLAY);
    CLEARCHECK(IDC_FLDNUM);
    CLEARCHECK(IDC_FLDPEN);
    return 0;
    } // msg CLEAR_FIELDINFO

  case CHANGED_CONN:
    // Changed connections (or status of current connection).
    SendMessage(hwnd, CHANGED_PS, 0, 0);      // Refresh if required
    return 0;

  case CHANGED_PS: {
    ECLFieldList  *Fields;      // Ptr to current session FieldList object
    char          Buff[20] = "Field n";

    // PS has been updated.  We only update the field display
    // if this is the current dialog tab (since it takes some
    // time to build the field list objects).
    if (CurrTab != 1)
      return 0;

    SendMessage(hwnd, CLEAR_FIELDINFO, 0, 0);  // Clear current GUI fields
    if (Session == NULL)                       // Stop if no current Sesion obj
      return 0;

    Fields = Session->GetPS()->GetFieldList(); // Get ptr to field list object
    Fields->Refresh();                         // Rebuild the list from current PS
    for (Fld = Fields->GetFirstField(), i=1;   // Insert each field in the listbox
         Fld != NULL;
         Fld = Fields->GetNextField(Fld), i++) {
      itoa(i, Buff+6, 10);
      SendDlgItemMessage(hwnd, IDC_FIELDLIST, LB_INSERTSTRING, (WPARAM)-1, (LPARAM)Buff);
    }
    return 0;

    } // CHANGED_PS

  case WM_COMMAND: 
    switch (LOWORD(mp1)) {
    case IDC_FIELDLIST: //------------------- Message from listbox
      if (HIWORD(mp1) == LBN_SELCHANGE) { //- User selected a field 
        // Process selected field of the listbox
        i = QUERYSELECTION(IDC_FIELDLIST);
        if (i<0)  // No selection
          return 0;

        // Walk field list to get to selected field
        Fld = Session->GetPS()->GetFieldList()->GetFirstField();
        for (; i>0; i--)
          Fld = Session->GetPS()->GetFieldList()->GetNextField(Fld);

        //------------- Field attributes
        if (Fld->IsModified())
          SETCHECK(IDC_FLDMOD);
        else
          CLEARCHECK(IDC_FLDMOD);

        if (Fld->IsProtected())
          SETCHECK(IDC_FLDPROTECT);
        else
          CLEARCHECK(IDC_FLDPROTECT);
        
        if (Fld->IsHighIntensity())
          SETCHECK(IDC_FLDHIGH);
        else
          CLEARCHECK(IDC_FLDHIGH);
        
        if (Fld->IsDisplay())
          SETCHECK(IDC_FLDDISPLAY);
        else
          CLEARCHECK(IDC_FLDDISPLAY);
        
        if (Fld->IsNumeric())
          SETCHECK(IDC_FLDNUM);
        else
          CLEARCHECK(IDC_FLDNUM);
        
        if (Fld->IsPenDetectable())
          SETCHECK(IDC_FLDPEN);
        else
          CLEARCHECK(IDC_FLDPEN);

        //------------- Field text
        char *Buff;
        Buff = new char[Fld->GetLength() + 1];
        Fld->GetScreen(Buff, Fld->GetLength() + 1, TextPlane);
        SETTEXT(IDC_FIELDTEXT, Buff);
        delete []Buff;

        //------------- Field position
        ULONG Srow, Scol, Erow, Ecol, Len;
        Fld->GetStart(&Srow, &Scol);
        Fld->GetEnd(&Erow, &Ecol);
        Len = Fld->GetLength();

        SetDlgItemInt(hwnd, IDC_FLDSTARTROW, Srow, FALSE);
        SetDlgItemInt(hwnd, IDC_FLDSTARTCOL, Scol, FALSE);
        SetDlgItemInt(hwnd, IDC_FLDENDROW,   Erow, FALSE);
        SetDlgItemInt(hwnd, IDC_FLDENDCOL,   Ecol, FALSE);
        SetDlgItemInt(hwnd, IDC_FLDLEN   ,   Len , FALSE);
      }
    }
    return FALSE;


  } // Switch on msg

  return FALSE;
}

//---------------------------------------------------------------------------------------------
BOOL WINAPI ConnDlgProc(HWND hwnd, UINT msg, WPARAM mp1, LPARAM mp2)
//---------------------------------------------------------------------------------------------
{

  switch (msg) {

  case WM_INITDIALOG: {
    break;
    }

  case WM_SHOWWINDOW:
    if ((BOOL)mp1 == TRUE) { // We are becoming visible
      SendMessage(hwnd, UPDATE_CONNINFO, 0, 0);  // Refresh GUI
    }
    break;

  case CHANGED_CONN:
    // Changed connections (or status of current connection).

    if (IsWindowVisible(hwnd))  // Only update if we are visible
      SendMessage(hwnd, UPDATE_CONNINFO, 0, 0);
    return 0;

  case UPDATE_CONNINFO:
    // Update GUI with current connection status
    char Buff[21];

    // Connection name
    Buff[0] = ConnList[CurrConn].ConnObj->GetName();
    Buff[1] = 0x00;
    SETTEXT(IDC_CONNNAME, Buff);

    // Connection handle
    sprintf(Buff, "0x%08lX", ConnList[CurrConn].ConnObj->GetHandle());
    SETTEXT(IDC_CONNHAND, Buff);

    // Connection type (converted to string)
    ECLBase::ConvertTypeToString(ConnList[CurrConn].ConnObj->GetConnType(), Buff);
    SETTEXT(IDC_CONNTYPE, Buff);

    // Host code page
    itoa(ConnList[CurrConn].ConnObj->GetCodePage(), Buff, 10);
    SETTEXT(IDC_CONNCP, Buff);

    // Ready flag
    if (ConnList[CurrConn].ConnObj->IsReady() == CONNSTATUS_READY)
      SETCHECK(IDC_CONNREADY);
    else
      CLEARCHECK(IDC_CONNREADY);

    // Started flag
    if (ConnList[CurrConn].ConnObj->IsStarted())
      SETCHECK(IDC_CONNSTARTED);
    else
      CLEARCHECK(IDC_CONNSTARTED);

    // Connected
    if (ConnList[CurrConn].ConnObj->IsCommStarted())
      SETCHECK(IDC_CONNCONNECTED);
    else
      CLEARCHECK(IDC_CONNCONNECTED);

    // API enabled
    if (ConnList[CurrConn].ConnObj->IsAPIEnabled())
      SETCHECK(IDC_CONNAPI);
    else
      CLEARCHECK(IDC_CONNAPI);

    return 0;

  } // Switch on msg

  return FALSE;
}

//---------------------------------------------------------------------------------------------
BOOL WINAPI MetricsDlgProc(HWND hwnd, UINT msg, WPARAM mp1, LPARAM mp2)
//---------------------------------------------------------------------------------------------
{

  switch (msg) {

  case WM_INITDIALOG: {
    break;
    }

  } // Switch on msg

  return FALSE;
}

//---------------------------------------------------------------------------------------------
BOOL WINAPI XferDlgProc(HWND hwnd, UINT msg, WPARAM mp1, LPARAM mp2)
//---------------------------------------------------------------------------------------------
{

  switch (msg) {

  case WM_INITDIALOG: {
    break;
    }

  } // Switch on msg

  return FALSE;
}

//---------------------------------------------------------------------------------------------
BOOL WINAPI DlgStartConnection(HWND hwnd, UINT msg, WPARAM mp1, LPARAM mp2)
//---------------------------------------------------------------------------------------------
// Dialog procedure for "Start Connection" dialog.
//---------------------------------------------------------------------------------------------
{
  static char ProfName[255] = "";  // Keep across instances of this dialog

  switch (msg) {

  case WM_INITDIALOG: {
    char Name[2];

    // Recall last used profile
    SETTEXT(IDC_STARTPROFILE, ProfName);
    // Set connection name on the GUI
    Name[0] = ConnList[CurrConn].ConnObj->GetName();
    Name[1] = 0x00;
    SETTEXT(IDC_STARTCONN, Name);

    SETCHECK(IDC_STARTREST);
    FOCUSCONTROL(IDC_STARTPROFILE);
    break;
    }

  case WM_COMMAND:
    switch (LOWORD(mp1)) {
    case IDOK: { //--------------------- (OK) Start connection button
      char *WinState;
      char Buff[sizeof(ProfName)+50];

      GETTEXT(IDC_STARTPROFILE, ProfName, sizeof(ProfName));
      if ((ProfName[0] == 0x00) || (ProfName[0] == ' ')) {
        ERRBOX(hwnd, "Invalid Profile Name","You must enter a workstation profile name with or without a '.WS' file type.");
        return 0;
      }
      if (QUERYCHECK(IDC_STARTMIN))
        WinState = "min";
      else if (QUERYCHECK(IDC_STARTMAX))
        WinState = "max";
      else if (QUERYCHECK(IDC_STARTMAX))
        WinState = "max";
      else if (QUERYCHECK(IDC_STARTREST))
        WinState = "restore";
      else if (QUERYCHECK(IDC_STARTHIDE))
        WinState = "hide";

      // Build ECLConnMgr::StartConnection config string
      sprintf(Buff, "profile=\"%s\" connname=%c winstate=%s",
              ProfName,
              ConnList[CurrConn].ConnObj->GetName(),
              WinState);

      // Start the new connection.  Note that control returns
      // here immediately and the connection will start sometime
      // later (our StartEvent object will be notified).

      try {
        ConnMgr->StartConnection(Buff);
      }
      catch (ECLErr Err) {
        ERRBOX(NULL, "Connection start failed.", Err.GetMsgText());
      }

      EndDialog(hwnd, IDOK);  // End this dialog
      return 0;
    } // IDOK

    case IDCANCEL:
      EndDialog(hwnd, IDCANCEL);
      return 0;

    } // switch on command ID

    return FALSE;

  } // Switch on msg

  return FALSE;
}
  
/*----------------------------------------------------------------------------*/
void DrawSessionButton(DRAWITEMSTRUCT *DrawData, LONG Color, LONG ColorBorder, char SessID)
/*----------------------------------------------------------------------------*/
/* Draw one of the connection buttons.  If it is the current connection,      */
/* hilight it with an inset double-border.                                    */
/*----------------------------------------------------------------------------*/
{
HBRUSH NewBrush, OldBrush;
HPEN   NewPen,   OldPen;

  NewBrush = CreateSolidBrush(Color);
  NewPen   = CreatePen(PS_SOLID, 1, CLR_DARKGRAY);

  if ((NewBrush) && (NewPen)) {
    OldBrush = (HBRUSH)SelectObject(DrawData->hDC, NewBrush);
    OldPen   = (HPEN)SelectObject(DrawData->hDC, NewPen);

    Rectangle(DrawData->hDC,       
             (DrawData->rcItem).left, (DrawData->rcItem).top,
             (DrawData->rcItem).right,(DrawData->rcItem).bottom);

    SetBkMode(DrawData->hDC, TRANSPARENT);
    SelectObject(DrawData->hDC, OldPen);
    DeleteObject(NewPen);
    NewPen = CreatePen(PS_SOLID, 1, CLR_BLACK);
    SelectObject(DrawData->hDC, NewPen);
    if ((SessID - 'A') == CurrConn) { // Hilight current connection
      int i;
      for (i=0; i<2; i++) {
      MoveToEx(DrawData->hDC, (DrawData->rcItem).left, (DrawData->rcItem).bottom, NULL);
      LineTo(DrawData->hDC, (DrawData->rcItem).left, (DrawData->rcItem).top);
      LineTo(DrawData->hDC, (DrawData->rcItem).right, (DrawData->rcItem).top);
      SelectObject(DrawData->hDC, OldPen);
      DeleteObject(NewPen);
      NewPen = CreatePen(PS_SOLID, 1, CLR_WHITE);
      SelectObject(DrawData->hDC, NewPen);
      InflateRect(&(DrawData->rcItem), -1, -1);
      MoveToEx(DrawData->hDC, (DrawData->rcItem).right, (DrawData->rcItem).top, NULL);
      LineTo(DrawData->hDC, (DrawData->rcItem).right, (DrawData->rcItem).bottom);
      LineTo(DrawData->hDC, (DrawData->rcItem).left, (DrawData->rcItem).bottom);
      SelectObject(DrawData->hDC, OldPen);
      DeleteObject(NewPen);
      NewPen = CreatePen(PS_SOLID, 1, CLR_BLACK);
      SelectObject(DrawData->hDC, NewPen);
      }
    }

    DrawText(DrawData->hDC, &SessID, 1, &(DrawData->rcItem), DT_CENTER|DT_VCENTER|DT_SINGLELINE);

    SelectObject(DrawData->hDC,OldPen);
    SelectObject(DrawData->hDC,OldBrush);
    DeleteObject(NewPen);
    DeleteObject(NewBrush);
  }

}

//---------------------------------------------------------------------------------------------
int WINAPI WinMain(HINSTANCE hinstInst, 
                   HINSTANCE hinstPrevInstance, 
                   LPSTR lpszCmdLine, 
                   int nCmdShow)
//---------------------------------------------------------------------------------------------
//  Main routine                                                                   
//---------------------------------------------------------------------------------------------
{
LOGFONT lFont;


   //Save app instance for later use
   hInst = hinstInst;

   InitCommonControls();

   // Make 6 point monospaced font
   lFont.lfHeight = -MulDiv(6, GetDeviceCaps(GetDC(GetDesktopWindow()), LOGPIXELSY), 72);
   lFont.lfWidth = 0;
   lFont.lfEscapement = 0;
   lFont.lfOrientation = 0;
   lFont.lfWeight = FW_NORMAL;
   lFont.lfItalic = 0;
   lFont.lfUnderline = 0;
   lFont.lfStrikeOut = 0;
   lFont.lfCharSet = ANSI_CHARSET;
   lFont.lfOutPrecision = OUT_DEFAULT_PRECIS;
   lFont.lfClipPrecision = CLIP_DEFAULT_PRECIS;
   lFont.lfQuality = DEFAULT_QUALITY;
   lFont.lfPitchAndFamily = FIXED_PITCH | FF_MODERN;
   strcpy(lFont.lfFaceName, "Courier New");
   MonoFont = CreateFontIndirect(&lFont);

 
   // To set a dialog control font:
   //SendDlgItemMessage(hwnd, CTR_ID, WM_SETFONT, (WPARAM)MonoFont, (LPARAM)TRUE);

   // The main dialog is the main application window.
   DialogBox(hinstInst, MAKEINTRESOURCE(IDD_MAIN), NULL, (DLGPROC)DlgMainProc);

   DeleteObject(MonoFont);

   return(0);
}
